/******************************************************************************/
/*  ZL5ARM - CAN                                                              */
/******************************************************************************/
                  
#include <LPC21xx.H>	    
#include "const_bit.h"
#include "lcd.h"

#define del_l 0x400000
#define del_s 0x10000

unsigned  int StandardFilter[2] 		_at_ 0xE0038000;	// Declare the standard acceptance filter table 
unsigned  int GroupStdFilter[2] 		_at_ 0xE0038008;
unsigned  int IndividualExtFilter[2]    _at_ 0xE0038010;
unsigned  int GroupExtFilter[2] 		_at_ 0xE0038018;

void delay (unsigned int del_time);
void LCD_Hex(int hex);
void CAN1IRQ(void)  __irq;
void init_CAN(void);

int main (void)
{
  unsigned int val;

  IODIR0 = 0xFF000000; 
  IODIR1 = 0x00FF0000;

  PINSEL1 &= !(BIT26 | BIT27 | BIT28 | BIT29);
  PINSEL1 |= BIT22;

  LCDInit();

  LCDSendByte(0x01,0);				// clrscr
  delay(del_s);

  LCDSendByte(' ',1);  delay(del_l);
  LCDSendByte(' ',1);  delay(del_l);
  LCDSendByte(' ',1);  delay(del_l);
  LCDSendByte('*',1);  delay(del_l);
  LCDSendByte(' ',1);  delay(del_l);
  LCDSendByte('Z',1);  delay(del_l);
  LCDSendByte('L',1);  delay(del_l);
  LCDSendByte('5',1);  delay(del_l);
  LCDSendByte('A',1);  delay(del_l);
  LCDSendByte('R',1);  delay(del_l);
  LCDSendByte('M',1);  delay(del_l);
  LCDSendByte(' ',1);  delay(del_l);
  LCDSendByte('*',1);  delay(del_l);
  delay(del_l);

  LCDTextXY(0, 1, " * www.btc.pl *");
  delay(del_l);
      		 
  LCDTextXY(0, 1, "    CAN->CAN   ");  
  delay(2*del_l);

  init_CAN();

  while(1)
  {
    C2TFI1 = 0x00040000;			// Set DLC to 4 bytes 	
    C2TID1 = 0x00000002;			// Set address to 2 Standard Frame

    do
    {
      val = ADDR;                            // Read A/D Data Register
    }	while ((val & 0x80000000) == 0);     // Wait for end of A/D Conversion

    if(C2SR & 0x00000004)					// See if Tx Buffer 1 is free
    {
      C2TDA1 = val;					// Copy A/D result into first four bytes
      C2CMR  = 0x00000021;			// Transmit the message
    }

    val = (val >> 6) & 0x03FF;
    LCDTextXY(0,1, "T=0x");
    LCD_Hex((val >> 8) & 0x0F);		// 1 hex digit
    LCD_Hex((val >> 4) & 0x0F);		// 2 hex digit
    LCD_Hex(val & 0x0F);			// 3 hex digit
  } // while


}
/////////////////////////////////////////////////////////////

void delay (unsigned int del_time) 
{ 
  unsigned int i;
  
  for(i = 0; i < del_time; i++);	   
}				    


void LCD_Hex(int hex)
{
  if (hex > 9) LCDSendByte('A' + (hex - 10),1);
  else         LCDSendByte('0' +  hex,1);
}

void CAN1IRQ (void) __irq
{
  unsigned int val;

  val = C1RDA;
  val = (val >> 6) & 0x03FF;

  LCDTextXY(9,1, "R=0x");
  LCD_Hex((val >> 8) & 0x0F);				// 1 hex digit
  LCD_Hex((val >> 4) & 0x0F);				// 2 hex digit
  LCD_Hex(val & 0x0F);						// 3 hex digit

  C1CMR = 0x00000004; 						// Release the recieve buffer
  VICVectAddr = 0x00000000;					// Signal the end of interrupt
}


void init_CAN(void)
{
  ADCR   			=  0x00270601; 			// A/D: 10-bit AIN0 @ 3MHz 
  ADCR 				|= 0x01000000;			// Start A/D Conversion 

  PINSEL1			|= 0x00014000;			// Enable P0.25 as CAN1 RX
  C2MOD				=  0x00000001;			// Set CAN controller into reset
  C2BTR				=  0x001C001D;			// Set bit timing to 125k
  C2MOD				=  0x00000000;			// Release CAN controller

  VPBDIV			=  0x00000001;			// Set PClk to 60Mhz
  IODIR1			=  0x00FF0000; 			// Set all ports to output
  PINSEL1			|= 0x00040000;			// Enable P0.25 as CAN1 RX
  C1MOD				=  0x00000001;			// Set CAN controller into reset
  C1BTR				=  0x001C001D;			// Set bit timing to 125k
  C1IER				=  0x00000001;			// Enable the Receive interrupt 
  VICVectCntl0		=  0x0000003A;  		// Select a priority slot for a given interrupt
  VICVectAddr0		=  (unsigned)CAN1IRQ;	// Pass the address of the IRQ into the VIC slot
  VICIntEnable		=  0x04000000;			// Enable interrupt
  AFMR 				=  0x00000001;			// Disable the Acceptance filters to allow setup of the table
  StandardFilter[0]	=  0x20012002;
  StandardFilter[1]	=  0x20032004;
  GroupStdFilter[0]	=  0x2009200F;
  GroupStdFilter[1]	=  0x20112020;
  IndividualExtFilter[0] = 0x40010000;
  IndividualExtFilter[1] = 0x40020000;
  GroupExtFilter[0]	=  0x40030000;
  GroupExtFilter[2]	=  0x40040000;
  SFF_sa			=  0x00000000;			// Set start address of Standard table
  SFF_GRP_sa		=  0x00000010;			// Set start address of Standard group table
  EFF_sa			=  0x00000018;			// Set start address of Extended table
  EFF_GRP_sa		=  0x00000020;			// Set start address of Extended group table
  ENDofTable		=  0x00000028;			// Set end of table address
  AFMR				=  0x00000000;			// Enable Acceptance filters

  C1MOD 			=  0x00000000;			// Release CAN controller
}
